#include <iostream>
#include <fstream>

#include <gtk/gtk.h>

#include "profilemanager/profileselector.h"
#include "profilemanager/intentselector.h"
#include "miscwidgets/patheditor.h"
#include "miscwidgets/generaldialogs.h"
#include "support/progressbar.h"
#include "imageutils/tiffsave.h"
#include "imageutils/jpegsave.h"
#include "miscwidgets/pixbufview.h"
#include "imagesource/imagesource_cms.h"
#include "imagesource/imagesource_util.h"
#include "imagesource/pixbuf_from_imagesource.h"

#include "menu_file.h"
#include "menu_edit.h"

#include "viewingcondselector.h"
#include "dialogs.h"
#include "util.h"

#include "it_gui.h"

#include "config.h"
#include "gettext.h"
#define _(x) gettext(x)

using namespace std;

// Simple class deriving from ConfigDB, to store UI settings such as
// the window geometry, etc.

IT_GUISettings::IT_GUISettings(ConfigFile *myfile) : ConfigDB(Template)
{
	new ConfigDBHandler(myfile,"[UISettings]",this);
}

IT_GUISettings::~IT_GUISettings()
{
}


ConfigTemplate IT_GUISettings::Template[]=
{
	ConfigTemplate("Win_X",int(30)),
	ConfigTemplate("Win_Y",int(30)),
	ConfigTemplate("Win_W",int(600)),
	ConfigTemplate("Win_H",int(400)),
	ConfigTemplate() // NULL terminated...
};


// Main GUI class - derives from IT_Core, handles window_creation and
// rendering of preview images.

IT_GUI::IT_GUI() : IT_Core(), window(NULL), factory(NULL), uisettings(this), filename(NULL),
	tmpsrcimage(NULL), devicelink(NULL), preview_original(NULL), preview_proof1(NULL), preview_dltrans(NULL)
{
	factory=GetTransformFactory();
}


IT_GUI::~IT_GUI()
{
	if(tmpsrcimage)
		delete tmpsrcimage;
	if(factory)
		delete factory;
	if(preview_original)
		g_object_unref(G_OBJECT(preview_original));
	if(preview_dltrans)
		g_object_unref(G_OBJECT(preview_dltrans));
	if(preview_proof1)
		g_object_unref(G_OBJECT(preview_proof1));
	if(filename)
		free(filename);
}


// Drag-and-drop
#define TARGET_URI_LIST 1
const static GtkTargetEntry dnd_file_drop_types[] = {
	{ "text/uri-list", 0, TARGET_URI_LIST }
};
static gint dnd_file_drop_types_count = 1;

static void get_dnd_data(GtkWidget *widget, GdkDragContext *context,
				     gint x, gint y,
				     GtkSelectionData *selection_data, guint info,
				     guint time, gpointer data)
{
	gchar *urilist=g_strdup((const gchar *)selection_data->data);
	IT_GUI *gui=(IT_GUI*)data;

	while(*urilist)
	{
		if(strncmp(urilist,"file:",5))
		{
			g_print("Warning: only local files (file://) are currently supported\n");
			while(*urilist && *urilist!='\n' && *urilist!='\r')
				++urilist;
			while(*urilist=='\n' || *urilist=='\r')
				*urilist++;
		}
		else
		{
			gchar *uri=urilist;
			while(*urilist && *urilist!='\n' && *urilist!='\r')
				++urilist;
			if(*urilist)
			{
				while(*urilist=='\n' || *urilist=='\r')
					*urilist++=0;
				gchar *filename=g_filename_from_uri(uri,NULL,NULL);
				gui->Load(filename);
				free(filename);
				gtk_drag_finish(context,TRUE,FALSE,time);
				return;
			}
		}
	}
	gtk_drag_finish(context,FALSE,FALSE,time);
}


// Source and destination profile selectors.


void IT_GUI::pbview_changed(GtkWidget *widget,gpointer user_data)
{
	// Synchronise the scale flag and offsets of the three pixbufviews.
	IT_GUI *gui=(IT_GUI *)user_data;
	PixbufView *pb=PIXBUFVIEW(widget);
	bool scale=pixbufview_get_scale(pb);
	int x=pixbufview_get_xoffset(pb);
	int y=pixbufview_get_yoffset(pb);

	pixbufview_set_scale(PIXBUFVIEW(gui->preview1),scale);
	pixbufview_set_scale(PIXBUFVIEW(gui->preview2),scale);
	pixbufview_set_scale(PIXBUFVIEW(gui->preview3),scale);
	pixbufview_set_offset(PIXBUFVIEW(gui->preview1),x,y);
	pixbufview_set_offset(PIXBUFVIEW(gui->preview2),x,y);
	pixbufview_set_offset(PIXBUFVIEW(gui->preview3),x,y);
}


void IT_GUI::destps_changed(GtkWidget *widget,gpointer user_data)
{
	IT_GUI *gui=(IT_GUI *)user_data;
	gui->factory->Flush();
	ProgressBar progress(_("Updating Proof..."),false,gui->window);
	gui->ProofImage(&progress);
	gui->OutputChanged();
}


void IT_GUI::srcps_changed(GtkWidget *widget,gpointer user_data)
{
	IT_GUI *gui=(IT_GUI *)user_data;
	gui->factory->Flush();
	ProgressBar progress(_("Updating previews..."),false,gui->window);
	gui->SrcProfileChanged();
	gui->RenderImage(&progress);
}


void IT_GUI::generatedl_clicked(GtkWidget *widget,gpointer user_data)
{
	IT_GUI *gui=(IT_GUI *)user_data;
	gui->GenerateDL();
}


void IT_GUI::SaveGeometry()
{
	if(window)
	{
		gint x,y,w,h;
		gtk_window_get_position(GTK_WINDOW(window),&x,&y);
		gtk_window_get_size(GTK_WINDOW(window),&w,&h);

		uisettings.SetInt("Win_X",x);
		uisettings.SetInt("Win_Y",y);
		uisettings.SetInt("Win_W",w);
		uisettings.SetInt("Win_H",h);
	}
}


// Render a proof using standard "dumb" colour transformation.

void IT_GUI::ProofImage(Progress *prog)
{
	if(!filename)
		return;
	try
	{
		ImageSource *is=ISLoadImage(filename);

		const char *target=profileselector_get_filename(PROFILESELECTOR(destprofselector));
		if(!target)
			throw _("Please set a suitable Target Device profile");
		cerr << "Got target profile: " << long(target) << ", " << target << endl;
		CMSProfile *p=GetProfile(target);
		CMSProfile *m=GetProfile(CM_COLOURDEVICE_DISPLAY);
		if(preview_proof1)
			g_object_unref(G_OBJECT(preview_proof1));
		preview_proof1=NULL;

		CMSProfile *src=is->GetEmbeddedProfile();
		if(!src)
			src=GetProfile(profileselector_get_filename(PROFILESELECTOR(srcprofselector)));

		CMSTransform *t=NULL;
		if((t=factory->GetTransform(m,src,p,
			LCMSWRAPPER_INTENT_PERCEPTUAL,LCMSWRAPPER_INTENT_RELATIVE_COLORIMETRIC)))
			is=new ImageSource_CMS(is,t);

//		if((t=factory->GetTransform(p,src,LCMSWRAPPER_INTENT_PERCEPTUAL)))
//			is=new ImageSource_CMS(is,t);
//		if((t=factory->GetTransform(m,p,LCMSWRAPPER_INTENT_RELATIVE_COLORIMETRIC)))
//			is=new ImageSource_CMS(is,t);

		if(prog)
			prog->SetMessage(_("Rendering normal proof..."));
		preview_proof1=pixbuf_from_imagesource(is,255,255,255,prog);

		delete is;
		if(m)
			delete m;
		if(p)
			delete p;
		pixbufview_set_pixbuf(PIXBUFVIEW(preview2),preview_proof1);
	}
	catch(const char *err)
	{
		ErrorMessage_Dialog(err,window);
	}
}


// Render a preview of the original image, using a straight
// Image -> Monitor transform


void IT_GUI::RenderImage(Progress *prog)
{
	if(!filename)
		return;
	try
	{
		ImageSource *is=ISLoadImage(filename);

		if(preview_original)
			g_object_unref(G_OBJECT(preview_original));
		preview_original=NULL;

		CMSProfile *src=is->GetEmbeddedProfile();
		CMSTransform *t=NULL;
		if(src)
			t=factory->GetTransform(CM_COLOURDEVICE_DISPLAY,src);
		else
		{
			src=GetProfile(profileselector_get_filename(PROFILESELECTOR(srcprofselector)));
			t=factory->GetTransform(CM_COLOURDEVICE_DISPLAY,src);
		}
		if(t)
			is=new ImageSource_CMS(is,t);

		preview_original=pixbuf_from_imagesource(is,255,255,255,prog);
		delete is;
		if(prog)
			prog->SetMessage(_("Rendering preview..."));
		pixbufview_set_pixbuf(PIXBUFVIEW(preview1),preview_original);
		ProofImage(prog);
	}
	catch(const char *err)
	{
		ErrorMessage_Dialog(err,window);
	}
}


// Render a proof of the image (which has already been transformed
// by the newly-generated devicelink profile), using a regular
// image -> monitor transform.

void IT_GUI::RenderDLImage(Temp_Image *img)
{
	try
	{
		ImageSource *is=ISLoadImage(img->Filename());

		if(preview_dltrans)
			g_object_unref(G_OBJECT(preview_dltrans));
		preview_dltrans=NULL;

		// Since we generated this image, we know it has an embedded profile...
		CMSProfile *src=is->GetEmbeddedProfile();
		CMSTransform *t=NULL;
		if(src)
			t=factory->GetTransform(CM_COLOURDEVICE_DISPLAY,src,LCMSWRAPPER_INTENT_RELATIVE_COLORIMETRIC);
		else
			throw _("Internal error: Couldn't find embedded profile!");

		if(t)
			is=new ImageSource_CMS(is,t);
		else
			throw _("Couldn't generate transform!");

		preview_dltrans=pixbuf_from_imagesource(is);
		delete is;
		pixbufview_set_pixbuf(PIXBUFVIEW(preview3),preview_dltrans);
	}
	catch(const char *err)
	{
		ErrorMessage_Dialog(err,window);
	}
}


// Load a new image.


void IT_GUI::Load(const char *fn)
{
	// Discard any previous image;
	if(tmpsrcimage)
		delete tmpsrcimage;
	tmpsrcimage=NULL;
	if(filename)
		free(filename);

	filename=strdup(fn);

	ProgressBar progress(_("Loading image..."),false,window);

	try
	{
		ImageSource *is=ISLoadImage(filename);

		// Compare the colourspace of the new image with the source profile selector.
		// If they vary, change the selector to match.

		CMSProfile *p=GetProfile(profileselector_get_filename(PROFILESELECTOR(srcprofselector)));
		if(p && p->GetColourSpace()!=STRIP_ALPHA(is->type))
		{
			profileselector_set_type(PROFILESELECTOR(srcprofselector),IS_TYPE(STRIP_ALPHA(is->type)));
			switch(STRIP_ALPHA(is->type))
			{
				case IS_TYPE_RGB:
					profileselector_set_filename(PROFILESELECTOR(srcprofselector),FindString("DefaultRGBProfile"));
					break;
				case IS_TYPE_CMYK:
					profileselector_set_filename(PROFILESELECTOR(srcprofselector),FindString("DefaultCMYKProfile"));
					break;
				default:
					throw _("Unsupported colour space");
			}
		}
		delete p;				

		// If the image has an embedded profile we honour it.

		p=is->GetEmbeddedProfile();
		if(p)
		{
			if(p->IsV4())
			{
				ErrorMessage_Dialog(_("Ignoring embedded profile -\nICC Version 4 Profiles not yet supported"));
				is->SetEmbeddedProfile(NULL);
				p=NULL;
			}
		}
		if(is->GetEmbeddedProfile())
			gtk_widget_set_sensitive(srcprofselector,FALSE);
		else
			gtk_widget_set_sensitive(srcprofselector,TRUE);
		delete is;

		RenderImage(&progress);
		
		if(devicelink)
			delete devicelink;
		devicelink=NULL;
		Update();
	}
	catch(const char *err)
	{
		ErrorMessage_Dialog(err,window);
	}
}


// Generate the DeviceLink.

void IT_GUI::GenerateDL()
{
	if(filename)
	{
		// Fetch the source and destination profiles from the UI, and set the
		// corresponding core variables.
		
		try
		{
			if(CreateDevicelinkDialog(GTK_WINDOW(window),*this))
			{
				if(devicelink)
					delete devicelink;
				devicelink=NULL;

				settings.SetString("SrcProfile",profileselector_get_filename(PROFILESELECTOR(srcprofselector)));
				settings.SetString("DstProfile",profileselector_get_filename(PROFILESELECTOR(destprofselector)));

				ProgressBar progress(_("Generating Devicelink..."),false,window);
				if(!tmpsrcimage)
					tmpsrcimage=new Temp_SrcImage(this,filename,&progress);
				devicelink=new Temp_DeviceLink(this,tmpsrcimage,&progress);
				Temp_Image conv_image(this,filename,devicelink,&progress);

				RenderDLImage(&conv_image);

				Update();
			}
		}
		catch(const char *err)
		{
			ErrorMessage_Dialog(err,window);
		}
	}
}


// Save functions

void IT_GUI::SaveTIFF()
{
	try
	{
		if(devicelink)
		{
			char *tmpfn=BuildFilename(filename,"_targetted","tif");
			char *fn=File_Save_Dialog(_("Please choose a filename..."),tmpfn,window);
			if(fn)
			{
				ProgressBar p(_("Saving TIFF..."),true,window);
				// Create transform from DeviceLink profile, and transform image.
				CMSProfile dl(devicelink->Filename());
				if(!dl.IsDeviceLink())
					throw _("Internal error: Generate profile isn't a devicelink!");

				CMSTransform trans(&dl);

				// Now load the image
				ImageSource *is=ISLoadImage(filename);

				is=new ImageSource_CMS(is,&trans);

				// We embed the provided output file in the targetted TIFF,
				// rather than the DeviceLink!
				CMSProfile *dstp=GetProfile(settings.FindString("DstProfile"));
				is->SetEmbeddedProfile(dstp);

				TIFFSaver ts(fn,is,16);
				ts.SetProgress(&p);
				ts.Save();
				delete is;
				delete dstp;
				free(fn);
			}
			free(tmpfn);
		}
		else
			ErrorMessage_Dialog(("Please generate a DeviceLink profile first!"));
	}
	catch(const char *err)
	{
		ErrorMessage_Dialog(("Please generate a DeviceLink profile first!"));
	}
}


void IT_GUI::SaveJPEG()
{
	try
	{
		if(devicelink)
		{
			char *tmpfn=BuildFilename(filename,"_targetted","jpg");
			char *fn=File_Save_Dialog(_("Please choose a filename..."),tmpfn,window);
			if(fn)
			{
				ProgressBar p(_("Saving TIFF..."),true,window);
				// Create transform from DeviceLink profile, and transform image.
				CMSProfile dl(devicelink->Filename());
				if(!dl.IsDeviceLink())
					throw _("Internal error: Generate profile isn't a devicelink!");

				CMSTransform trans(&dl);

				// Now load the image
				ImageSource *is=ISLoadImage(filename);

				is=new ImageSource_CMS(is,&trans);

				// We embed the provided output file in the targetted TIFF,
				// rather than the DeviceLink!
				CMSProfile *dstp=GetProfile(settings.FindString("DstProfile"));
				is->SetEmbeddedProfile(dstp);

				JPEGSaver ts(fn,is,95);
				ts.SetProgress(&p);
				ts.Save();
				delete is;
				delete dstp;
				free(fn);
			}
			free(tmpfn);
		}
		else
			ErrorMessage_Dialog(_("Please generate a DeviceLink profile first!"));
	}
	catch(const char *err)
	{
		ErrorMessage_Dialog(err);
	}
}


void IT_GUI::SaveDevicelink()
{
	if(devicelink)
	{
		char *tmpfn=BuildFilename(filename,"_devlink","icc");
		char *fn=File_Save_Dialog(_("Please choose a filename for the DeviceLink profile..."),tmpfn,window);
		if(fn)
		{
#define TMPBUFSIZE 4096
			char buffer[TMPBUFSIZE];
			ifstream in(devicelink->Filename(),ios::in|ios::binary);
			ofstream out(fn,ios::out|ios::binary);
			while(!in.eof())
			{
				in.read(buffer,TMPBUFSIZE);
				int n=in.gcount();
				if(n)
					out.write(buffer,n);
			}
			out.close();
			in.close();
			free(fn);
		}
		free(tmpfn);
	}
	else
		ErrorMessage_Dialog(("Please generate a DeviceLink profile first!"));
}


// GUI functions

// If the source viewing conditions are changed by the user, then a cached Gamut is no longer
// valid and must be regenerated.
void IT_GUI::GamutChanged()
{
	if(tmpsrcimage)
		tmpsrcimage->FlushGamut();
	if(devicelink)
		delete devicelink;
	devicelink=NULL;
	Update();
}


void IT_GUI::SrcProfileChanged()
{
	if(tmpsrcimage)
	{
		tmpsrcimage->FlushSrcProfile();
		tmpsrcimage->FlushGamut();
	}
	if(devicelink)
		delete devicelink;
	devicelink=NULL;
	Update();
}


// If the preprocessing (blur, etc) changes, then all cached data
// is invalidated.

void IT_GUI::PreprocessChanged()
{
	if(tmpsrcimage)
		delete tmpsrcimage;
	tmpsrcimage=NULL;
	if(devicelink)
		delete devicelink;
	devicelink=NULL;
	Update();
}


// If either the output viewing conditions or the target profile are changed, then
// the devicelink must be regenerated.

void IT_GUI::OutputChanged()
{
	if(devicelink)
		delete devicelink;
	devicelink=NULL;
	Update();
}


void IT_GUI::Update()
{
	// Has a change been made which invalidates the DeviceLink profile?
	if(!devicelink)
	{
		if(preview_dltrans)
		{
			cerr << "Clearing preview3 pixbuf" << endl;
			pixbufview_set_pixbuf(PIXBUFVIEW(preview3),NULL);
			g_object_unref(G_OBJECT(preview_dltrans));
			preview_dltrans=NULL;
		}
	}
	if(filename)
	{
		gtk_widget_set_sensitive(generatebutton,TRUE);
	}
	GtkWidget *tmp;
	tmp=gtk_ui_manager_get_widget(uimanager,"/MainMenu/FileMenu/SaveDeviceLink");
	gtk_widget_set_sensitive(tmp,(devicelink!=NULL));
	tmp=gtk_ui_manager_get_widget(uimanager,"/MainMenu/FileMenu/SaveTIFF");
	gtk_widget_set_sensitive(tmp,(devicelink!=NULL));
	tmp=gtk_ui_manager_get_widget(uimanager,"/MainMenu/FileMenu/SaveJPEG");
	gtk_widget_set_sensitive(tmp,(devicelink!=NULL));
}


#define APPLYDND(widget) \
	gtk_drag_dest_set(GTK_WIDGET(widget), \
		GtkDestDefaults(GTK_DEST_DEFAULT_MOTION | GTK_DEST_DEFAULT_DROP), \
		dnd_file_drop_types, dnd_file_drop_types_count, \
		GdkDragAction(GDK_ACTION_COPY)); \
	g_signal_connect(G_OBJECT(widget), "drag_data_received", \
		G_CALLBACK(get_dnd_data), this);


void IT_GUI::Run()
{
	window=gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title (GTK_WINDOW (window), PACKAGE_STRING);
	gtk_signal_connect (GTK_OBJECT (window), "delete_event",
		(GtkSignalFunc) gtk_main_quit, NULL);
	gtk_window_set_default_size (GTK_WINDOW (window), uisettings.FindInt("Win_W"),uisettings.FindInt("Win_H"));
	gtk_window_move(GTK_WINDOW(window),uisettings.FindInt("Win_X"),uisettings.FindInt("Win_Y"));

	GtkWidget *vbox = gtk_vbox_new (FALSE, 0);
	gtk_container_add (GTK_CONTAINER (window), vbox);
	gtk_widget_show (vbox);

	GtkWidget *label;

	// Drag-and-drop
	
	APPLYDND(vbox);

	// Menus

	GtkWidget *menubar;
	GtkAccelGroup *accel_group;
	uimanager = gtk_ui_manager_new ();

	BuildFileMenu(this,uimanager);
	BuildEditMenu(this,uimanager);
	
	accel_group = gtk_ui_manager_get_accel_group (uimanager);
	gtk_window_add_accel_group (GTK_WINDOW (window), accel_group);

	menubar = gtk_ui_manager_get_widget (uimanager, "/MainMenu");
	gtk_box_pack_start(GTK_BOX(vbox),menubar,FALSE,TRUE,0);
	gtk_widget_show(menubar);


	// Previews

	GtkWidget *notebook=gtk_notebook_new();
	gtk_box_pack_start(GTK_BOX(vbox),notebook,TRUE,TRUE,0);
	gtk_widget_show(GTK_WIDGET(notebook));

	label=gtk_label_new(_("Original image"));
	gtk_widget_show(label);

	preview1=pixbufview_new(NULL);
	g_signal_connect(preview1,"changed",G_CALLBACK(pbview_changed),this);
	gtk_notebook_append_page(GTK_NOTEBOOK(notebook),GTK_WIDGET(preview1),GTK_WIDGET(label));
	gtk_widget_show(preview1);

	APPLYDND(preview1);


	label=gtk_label_new(_("Normal proof"));
	gtk_widget_show(label);

	preview2=pixbufview_new(NULL);
	g_signal_connect(preview2,"changed",G_CALLBACK(pbview_changed),this);
	gtk_notebook_append_page(GTK_NOTEBOOK(notebook),GTK_WIDGET(preview2),GTK_WIDGET(label));
	gtk_widget_show(preview2);

	APPLYDND(preview2);


	label=gtk_label_new(_("Proof with custom DeviceLink"));
	gtk_widget_show(label);

	GtkWidget *vbox2=gtk_vbox_new(FALSE,0);
	gtk_notebook_append_page(GTK_NOTEBOOK(notebook),GTK_WIDGET(vbox2),GTK_WIDGET(label));
	gtk_widget_show(vbox2);

	preview3=pixbufview_new(NULL);
	g_signal_connect(preview3,"changed",G_CALLBACK(pbview_changed),this);
	gtk_box_pack_start(GTK_BOX(vbox2),GTK_WIDGET(preview3),TRUE,TRUE,0);
	gtk_widget_show(preview3);
	APPLYDND(preview3);


	// Profiles and Viewing Conditions

	GtkWidget *table=gtk_table_new(3,4,FALSE);
	gtk_box_pack_start(GTK_BOX(vbox),table,FALSE,FALSE,0);
	gtk_container_set_border_width(GTK_CONTAINER(table),8);
	gtk_table_set_col_spacing(GTK_TABLE(table),1,16);
	gtk_table_set_row_spacings(GTK_TABLE(table),4);
	gtk_widget_show(table);
	
	label=gtk_label_new(_("Source Profile:"));
	gtk_table_attach_defaults(GTK_TABLE(table),label,0,1,0,1);
	gtk_widget_show(label);
		
	srcprofselector = profileselector_new(this,IS_TYPE_RGB);
	profileselector_set_filename(PROFILESELECTOR(srcprofselector),settings.FindString("SrcProfile"));
	g_signal_connect(srcprofselector,"changed",G_CALLBACK(srcps_changed),this);
	gtk_table_attach_defaults(GTK_TABLE(table),srcprofselector,1,2,0,1);
	gtk_widget_show(srcprofselector);

	label=gtk_label_new(_("Target Profile:"));
	gtk_table_attach_defaults(GTK_TABLE(table),label,0,1,1,2);
	gtk_widget_show(label);

	destprofselector = profileselector_new(this);
	profileselector_set_filename(PROFILESELECTOR(destprofselector),settings.FindString("DstProfile"));
	g_signal_connect(destprofselector,"changed",G_CALLBACK(destps_changed),this);
	gtk_table_attach_defaults(GTK_TABLE(table),destprofselector,1,2,1,2);
	gtk_widget_show(destprofselector);

	generatebutton=gtk_button_new_with_label(_("Generate optimal devicelink..."));
	g_signal_connect(generatebutton,"clicked",G_CALLBACK(generatedl_clicked),this);
	gtk_table_attach_defaults(GTK_TABLE(table),generatebutton,2,3,0,1);
	gtk_widget_show(generatebutton);

	gtk_widget_set_sensitive(generatebutton,FALSE);

	gtk_widget_show(window);

	Update();

	if(!CheckArgyllUtils())
	{
		ErrorMessage_Dialog(_("Argyll utilities (tiffgamut and collink) not found.\n"
		"Please ensure Argyll is installed (http://www.argyllcms.com)\n"
		"and then set the location of these two utilities\n"
		"in the Preferences dialog..."),window);
		PreferencesDialog(GTK_WINDOW(window),*this);
	}

	gtk_main();
}
